/*******************************************************************************
 * Constants used for quick access to the Components object.
 ******************************************************************************/
const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;
const kXULNS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
/*******************************************************************************
 * Main object that interacts between the user and the backend.
 ******************************************************************************/
var NewslivePanel = {

  _queue: null,
  get queue() { return this._queue; },
  set queue(val) { this._queue = val; },
  
  _timer: null,
  get timer() { return this._timer; },
  set timer(val) { this._timer = val; },

  _obService: null,
  get obService() { return this._obService; },
  set obService(val) { this._obService = val; },
  
  _view: null,
  get view() { return this._view; },
  set view(val) { this._view = val; },
      
  get tree() { return document.getElementById("newslive_tree"); },
  
  // nsISupports
  QueryInterface: function nlp_QI(iid) {
    if (iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsIController) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  }, 
  
  // nsIObserver
  observe: function nl_observe(subject, topic, data) {
  
    // push to queue
    this.queue[this.queue.length] = { "data": data, "subject": subject };
    
    // start the queue timer
    if (!this.timer) {
      var self = this;
      this.timer = window.setTimeout(function() { self.checkQueue(); }, 500);
    }
  },
  
  // nsIController
  supportsCommand: function nlp_supportsCommand(id) {
    switch (id) {
    case "newslive_open":
    case "newslive_openWin":
    case "newslive_openTab":
    case "newslive_openSidebar":
    case "newslive_openAll":
    case "newslive_reload":    
      return true;
     default:
       return false;
    }
  },
  
  isCommandEnabled: function nlp_isCommandEnabled(id) {
    var item = this.getSelected();
    var type = (!item) ? null : item.getAttribute("itemtype");
    switch (id) {
    case "newslive_open":
      return (type == "item");
    case "newslive_openWin":
      return (type == "item");
    case "newslive_openTab":
      return (type == "item");    
    case "newslive_openSidebar":
      return (type == "item");       
    case "newslive_openAll":
      return (type == "feed");    
    case "newslive_reload": 
      return (type == "feed");       
    default:
      return false;
    }
  },
  
  doCommand: function nlp_doCommand(id) {
    var item = this.getSelected();
    switch (id) {
    case "newslive_open":
      this.view.open(item, "current");
      break;  
    case "newslive_openWin":
      this.view.open(item, "window");
      break;      
    case "newslive_openTab":
      this.view.open(item, "tab");
      break;
    case "newslive_openSidebar":
      this.view.open(item, "sidebar");
      break;      
    case "newslive_openAll":
      this.view.openAll(item);
      break;
    case "newslive_reload": 
      this.view.reload(item);
      break;    
    default:
      return;
    }
  },
  
  onEvent: function nlp_onEvent(name) {
    goUpdateCommand("newslive_open");    
    goUpdateCommand("newslive_openWin");
    goUpdateCommand("newslive_openTab");  
    goUpdateCommand("newslive_openSidebar");     
    goUpdateCommand("newslive_openAll");  
    goUpdateCommand("newslive_reload");  
  },
    
  onLoad: function nlp_onLoad() {

    // setup the util object
    this.queue = [];
    var parentNode = document.getElementById("newslive_container");
    this.view = new NewsliveView("tree", parentNode, null);
    
    // hookup this as a controller
    this.tree.controllers.appendController(this);
    
    // get the observer service and add ourself as an observer
    this.obService = Cc["@mozilla.org/observer-service;1"].
                      getService(Ci.nsIObserverService);
    this.obService.addObserver(this, "netscape-newslive", false);
                     
    // load the tree and focus it
    var self = this;
    window.setTimeout(function() { self.loadDelayed(); self.tree.focus();}, 0);  
  },
  
  onUnload: function nlp_onUnload() {
  
    // stop the queue timer
    if (this.timer)
      window.clearTimeout(this.timer);
    
    // remove ourself as an observer
    this.obService.removeObserver(this, "netscape-newslive");
    
    // unhook the controller
    this.tree.controllers.removeController(this);
    
    // save the tree state
    this.persistOpen();
    
    // clean up the utility object
    this.view.parentNode = null;
    this.view.lastChild = null;
    this.view.type = null;

    // remove variables
    this.queue = null;
    this.timer = null; 
    this.obService = null;
    this.view = null;
  },
  
  
  checkQueue: function nlp_checkQueue() {   
    while (this.queue.length) {
      var item = this.queue.shift();
      try {
        this[item.data](item.subject);   
      } catch(e) {} 
    }
    this.timer = null;
  },
    
  loadDelayed: function nlp_loadDelayed() {
  
    // load the tree
    this.view.loadDelayed();
    
    // restore open states
    this.restoreOpen();
  },

  changedState: function nlp_changedState() {},
  sameFeeds: function nlp_sameFeeds() {},
  changedFeeds: function nlp_changedFeeds() {},
  newFeeds: function nlp_newFeeds() {
    this.loadDelayed();
  },
  addItem: function nlp_addItem(item) { this.view.addItem(item); },
  removeItem: function nlp_removeItem(item) { this.view.removeItem(item); },
  updateItem: function nlp_updateItem(item) { this.view.updateItem(item); },
  addItems: function nlp_addItems(items) { this.view.addItems(items); },
  removeItems: function nlp_removeItems(feed) { this.view.removeItems(feed); }, 
  
  openItemKey: function nlp_openItemKey() {
  
    // get the tree selection
    var target = this.getSelected();
    
    // nothing selected or not a valid target
    if (!target || !this.view.isValidTarget(target))
      return;
    
    // target is a feed do nothing
    var itemtype = target.getAttribute("itemtype");
    if (itemtype == "feed")
      return;
    
    // open the item
    var where = nsWhereToOpenLink(this.view.utils.domain + "open");
    this.view.open(target, where);
  },
  
  openItemClick: function nlp_openItemClick(event) {
  
    // click was not on a tree child
    if (event.originalTarget.localName != "treechildren")
      return;  
    
    // get the cell that was clicked      
    var row = {};
    var col = {};
    var obj = {};
    var box =  this.tree.boxObject.QueryInterface(Ci.nsITreeBoxObject);
    box.getCellAt(event.clientX, event.clientY, row, col, obj);

    // twisty or not a tree child clicked
    if (row.value == -1 || obj.value == "twisty")
      return;
    
    // get the tree selection
    var target = this.getSelected();

    // nothing selected or not a valid target    
    if (!target || !this.view.isValidTarget(target))
      return;

    // middle button clicked open item
    var itemtype = target.getAttribute("itemtype");
    if (event.button == 1)
      this.view.openItem(event, target);    
      
    // left button clicked
    else if (event.button === 0) {
    
      // open if item
      if (itemtype == "item")
        this.view.openItem(event, target);
        
      // toggle view state for feed  
      else
        this.tree.view.toggleOpenState(this.tree.view.getIndexOfItem(target));
    }
  },

  buildContextMenu: function nlp_buildContextMenu(event) {
  
    // setup variables
    var separator = null;
    var command = null;
    var visibleItemsBeforeSep = false;
    var anyVisible = false;
    
    // loop through the context menu
    var popup = event.target;
    for (var i=0; i<popup.childNodes.length; ++i) {
    
      // get the menuitem
      var item = popup.childNodes[i];
      
      // determine if item should be visible
      if (item.localName != "menuseparator") {
        command = document.getElementById(item.getAttribute("command"));
        item.hidden = command.hasAttribute("disabled");

        // item is visible mark variables as such
        if (!item.hidden) {
          visibleItemsBeforeSep = true;
          anyVisible = true;
 
          // show the separator above the menu-item if any
          if (separator) {
            separator.hidden = false;
            separator = null;
          }
        }
      }

      // menuseparator
      else {

        // initially hide it. It will be unhidden if there will be at least one
        // visible menu-item above and below it.
        item.hidden = true;

        // we won't show the separator at all if no items are visible above it
        if (visibleItemsBeforeSep)
          separator = item;
 
        // new separator, count again:
        visibleItemsBeforeSep = false;
      }
    }

    return anyVisible;
  }, 
    
  getSelected: function nlp_getSelected() {
  
    // get the tree view and range count
    var view = this.tree.view;
    var range = view.selection.getRangeCount();
    var items = [];
    
    // loop through the range
    for (var i=0; i<range; i++) {
    
      // return the item at the range position
      var start = {};
      var end = {};
      view.selection.getRangeAt(i, start, end);
      return view.getItemAtIndex(start.value);
    }
    
    // nothing selected return null
    return null;
  },
  
  persistOpen: function nlp_persistOpen() {
  
    // loop through the child nodes
    var elts = [];
    for (var i=0; i<this.view.parentNode.childNodes.length; i++) {
    
      // save the id if the item is open
      var item = this.view.parentNode.childNodes[i];
      if (item.getAttribute("open") == "true")
        elts.push(item.getAttribute("id"));
    }
    
    // persist the open elements on the tree
    this.tree.setAttribute("openelements", elts.join(","));
    document.persist(this.tree.getAttribute("id"), "openelements");
  },
  
  restoreOpen: function nlp_restoreOpen() {
  
    // get the list of open elements
    var elts = this.tree.getAttribute("openelements").split(","); 
    
    // loop through the child nodes
    for (var i=0; i<this.view.parentNode.childNodes.length; i++) {
      var item = this.view.parentNode.childNodes[i];
      var id = item.getAttribute("id");
      
      // item should be open
      if (elts.indexOf(id) != -1)
        item.setAttribute("open", "true");
        
      // item should not be open  
      else
        item.removeAttribute("open");
    }
  }
};