/*******************************************************************************
 * A utility object that can be used access different values.  
 ******************************************************************************/
function NewsliveUtils() {}
NewsliveUtils.prototype = {

  _branch: null,
  get domain() { return "extensions.netscape.newslive."; },
  get branch() { 
    if (!this._branch) {
      var pbService = Cc["@mozilla.org/preferences-service;1"].
                      getService(Ci.nsIPrefService);
      this._branch = pbService.getBranch(this.domain);
    }
    return this._branch; 
  },
  
  _nlService: null,  
  get nlService() {
    if (!this._nlService)
      this._nlService = Cc["@netscape.com/newslive/service;1"].
                        getService(Ci.nsINewsliveService);
    return this._nlService;                 
  },
    
  // nsISupports
  QueryInterface: function nlu_QI(iid) {
    if (iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },  
      
  getBranch: function nlu_getBranch(name) {
    if (name == undefined) 
      return this.branch;
      
    var pbService = Cc["@mozilla.org/preferences-service;1"].
                    getService(Ci.nsIPrefService);
    return pbService.getBranch(name);
  },
  
  getString: function nlu_getString(name, replace) { 
    var bundle = document.getElementById("newslive_bundle");
    if (!bundle)
      return null;
      
    if (!replace)
      return bundle.getString(name);
    else
      return bundle.getFormattedString(name, replace);
  },
  
  getOS: function nlu_getOS() {
    var app = Cc["@mozilla.org/xre/app-info;1"].
              getService(Ci.nsIXULAppInfo).
              QueryInterface(Ci.nsIXULRuntime);  
    return app.OS.toLowerCase();          
  },
  
  getURL: function nlu_getURL(target) {
    var redirect = this.branch.getCharPref("itemOpen");
    return (target.getAttribute("url") + redirect);
  },
    
  openManage: function nlu_openManage() {
    this.openWindow("newslive:manage", 
                    "chrome://browser/content/netscape-newslive/newslive-manage.xul", 
                    "chrome=yes,centerscreen=yes,dependent=no,dialog=yes");                 
  },
    
  /**
   * Based on toOpenWindowByType in browser.js
   */
  openWindow: function nlu_openWindow(type, url, features) {
    var winMgr = Cc['@mozilla.org/appshell/window-mediator;1'].
                 getService(Ci.nsIWindowMediator);
    var top = winMgr.getMostRecentWindow(type);

    if (top) {
      top.focus();
      return;
    }
  
    if (!features)
      features = "chrome,extrachrome,menubar,resizable,scrollbars,status,toolbar";
    window.open(url, "_blank", features);
  }
};

/*******************************************************************************
 * A utility object used to build views.
 ******************************************************************************/
function NewsliveView(type, parent, last) {
  this.utils = new NewsliveUtils(); 
  this.type = type;
  this.parentNode = parent;
  this.lastChild = last;
}
NewsliveView.prototype = {
  
  _utils: null,
  get utils() { return this._utils; },
  set utils(val) { this._utils = val; },
  
  _type: null,
  get type() { return this._type; },
  set type(val) { this._type = val;},
  
  _parentNode: null,
  get parentNode() { return this._parentNode; },
  set parentNode(val) { this._parentNode = val; },
  
  _lastChild: null,
  get lastChild() { return this._lastChild; },
  set lastChild(val) { this._lastChild = val; },
    
  loadDelayed: function nlv_loadDelayed() {
  
    // remove any previous children - stop if we hit the last child
    this.removeFeeds();
    
    // loop through subscribed feeds and add
    var feeds = this.utils.nlService.getItems("newslive_root", true);
    var index = 0;
    while (index < feeds.length) {
      var feed = feeds.queryElementAt(index, Ci.nsINewsliveItem);
      this.addFeed(feed);      
      index++;
    }  
  },
  
  isFeed: function nlv_isFeed(item) {
    return (item.parentId == "newslive_root");
  },
  
  addFeed: function nlv_addFeed(feed) {
    
    // do nothing if not subscribed
    if (!feed.subscribed)
      return;
    
    // remove the loading message from the parent
    this.removeLoading(this.parentNode);
    
    // add the feed node and add its items if the service has any items  
    var node = this.createNode(feed, this.parentNode);     
    if (this.utils.nlService.hasItems(feed.id, true)) {
      var items = this.utils.nlService.getItems(feed.id, false);
      this.addItems(items); 
      return;
    }
    
    // add the loading message to the feed
    this.addLoading(node, "items");  
  },
  
  removeFeed: function nlv_removeFeed(feed) {
   
    // get the feed node 
    var suffix = (this.type == "menu") ? "_menu" : "_treeitem";  
    var node = this.getNode(feed.id, suffix);
    if (!node)
      return;
      
    // remove the feed node and add the loading message to the parent
    this.removeNode(node);
    this.addLoading(this.parentNode, "feeds");
  },
  
  removeFeeds: function nlv_removeFeeds() {
  
    // remove any previous children - stop if we hit the last child
    var node = this.parentNode;
    while (node.lastChild != this.lastChild)
      this.removeNode(node.lastChild);
    
    // service does not have any feeds so add loading message    
    if (!this.utils.nlService.hasItems("newslive_root", true)) {
      this.addLoading(node, "feeds");
      return;
    }
  },
      
  updateFeed: function nlv_updateFeed(feed) {  
    
    // remove the feed if no longer subscribed
    if (!feed.subscribed) {  
      this.removeFeed(feed);
      return;
    }
    
    // get the feed node.  Add it if it does not exist
    var suffix = (this.type == "menu") ? "_menu" : "_treeitem";  
    var node = this.getNode(feed.id, suffix);
    if (!node) {
      this.addFeed(feed);
      return;
    }

    // update the node's attributes
    var sortIndex = Number(node.getAttribute("sortIndex"));    
    node.setAttribute("url", feed.URL);      
    node.setAttribute("sortIndex", String(feed.sortIndex));
    if (this.type == "menu") {  
      node.setAttribute("label", feed.title);
      node.setAttribute("image", feed.iconURL);
    } else {
      var cell = this.getNode(feed.id, "_treecell");
      cell.setAttribute("label", feed.title);
      cell.setAttribute("src", feed.iconURL);
    }
    
    // move the node if the sort index changed
    if (sortIndex != feed.sortIndex) {
      var parent = node.parentNode;
      this.removeNode(node);
      this.addNode(node, parent);
    }
  },
    
  addItem: function nlv_addItem(item) {
    item = item.QueryInterface(Ci.nsINewsliveItem);
    if (this.isFeed(item)) {
      this.addFeed(item);
      return;
    }
    
    // get the feed node.  Do nothing if the node does not exist
    var suffix = (this.type == "menu") ? "_menupopup" : "_treechildren";  
    var node = this.getNode(item.parentId, suffix);
    if (!node)
      return;
    
    // remove the loading and open in tabs messages
    this.removeLoading(node);
    this.removeOpenInTabs(node);
    
    // create the item node and add the loading and open in tabs messages  
    var child = this.createNode(item, node);
    this.addLoading(node, "items");
    this.addOpenInTabs(node);
  },
  
  removeItem: function nlv_removeItem(item) {
    item = item.QueryInterface(Ci.nsINewsliveItem);
    if (this.isFeed(item)) {
      this.removeFeed(item);
      return;
    }
    
    // get the feed node.  Do nothing if the node does not exist
    var suffix = (this.type == "menu") ? "_menupopup" : "_treechildren";  
    var node = this.getNode(item.parentId, suffix);
    if (!node)
      return;

    // remove the loading and open in tabs messages
    this.removeLoading(node);
    this.removeOpenInTabs(node);
    
    // get the item node.  Do nothing if the node does not exist
    suffix = (this.type == "menu") ? "_menuitem" : "_treeitem";
    var child = this.getNode(item.id, suffix);
    if (!child)
      return;
    
    // remove the node and add the loading and open in tabs messages 
    this.removeNode(child);
    this.addLoading(node, "items");
    this.addOpenInTabs(node);
  },
  
  updateItem: function nlv_updateItem(item) {
    item = item.QueryInterface(Ci.nsINewsliveItem);
    if (this.isFeed(item)) {
      this.updateFeed(item);
      return;
    }
      
    this.removeItem(item);
    this.addItem(item);
  },
  
  addItems: function nlv_addItems(items) {   
    items = items.QueryInterface(Ci.nsIArray);
    
    // loop through the items
    var node, suffix, child;
    var index = 0;
    while (index < items.length) {
      var item = items.queryElementAt(index, Ci.nsINewsliveItem);
      if (this.isFeed(item)) {
        this.addFeed(item);
        index++;
        continue;
      }
    
      // get the feed node if we do not have it
      if (!node) {
      
        // do nothing if the feed node does not exist
        suffix = (this.type == "menu") ? "_menupopup" : "_treechildren";  
        node = this.getNode(item.parentId, suffix);
        if (!node)
          return;

        // remove the loading and open in tabs messages
        this.removeLoading(node);
        this.removeOpenInTabs(node);  
      }
      
      // add the item node
      child = this.createNode(item, node);
      index++;
    }
    
    // do nothing if the feed node does not exist
    if (!node)
      return;
    
    // add the loading and open in tabs messages
    this.addLoading(node, "items");
    this.addOpenInTabs(node);
  },
    
  removeItems: function nlv_removeItems(feed) {
    if (!feed) {
      this.removeFeeds();
      return;
    } else
      feed = feed.QueryInterface(Ci.nsINewsliveItem);

    // get the feed node.  Do nothing if the node does not exist
    var suffix = (this.type == "menu") ? "_menupopup" : "_treechildren";  
    var node = this.getNode(feed.id, suffix);
    if (!node)
      return;
    
    // remove the item nodes
    while (node.hasChildNodes())
      this.removeNode(node.lastChild);

    // add the loading and open in tabs messages
    this.addLoading(node, "items");
    this.addOpenInTabs(node);
  },
  
  openItem: function nlv_openItem(event, target) {
    // get the target if not passed in
    if (!target)
      target = event.originalTarget;
      
    // do nothing if target disabled
    if (target.hasAttribute("disabled"))
      return;
    
    // change the target to the feed if it is the open in tabs item     
    if (target.getAttribute("class") == "openintabs-menuitem")
      target = target.parentNode.parentNode;

    // do nothing if not a valid target
    if (!this.isValidTarget(target))
      return;
    
    // determine where to open the link
    var where = nsWhereToOpenLink(this.utils.domain + "open", event);
    
    // open all if the target is a feed    
    if (target.getAttribute("itemtype") == "feed") 
      this.openAll(target);
      
    // open the item  
    else {
      openUILinkIn(this.utils.getURL(target), where, false);
      window.content.focus();
    }
              
    // stop the event
    event.stopPropagation();
  },
  
  openAll: function nlv_openAll(target) {
  
    // get the feed node
    var itemid = target.getAttribute("itemid");
    var suffix = (this.type == "menu") ? "_menupopup" : "_treechildren";
    var node = document.getElementById(itemid + suffix);
    
    // loop through it's child nodes
    var urls = [];
    for (var i=0; i<node.childNodes.length; i++) {
      
      // get the item node.  Do nothing if not a valid target
      target = node.childNodes[i];
      if (!this.isValidTarget(target))
        continue;
      
      // save the item url  
      urls.push(this.utils.getURL(target));  
    }   
    
    // open the urls  
    this.openURLs(urls, "tab");      
  },
   
  open: function nlv_open(target, where) { 
    var url = this.utils.getURL(target);
    var title = target.getAttribute("label");
    if (where == "sidebar") 
        openWebPanel(title, url);
    else {
      openUILinkIn(url, where, false);
      window.content.focus();
    }
  },
  
  reload: function nlv_reload(target) {
    var itemid = target.getAttribute("itemid");
    this.utils.nlService.reloadItems(itemid);  
  },
    
  isValidTarget: function nlv_isValidTarget(target) {
    return target.hasAttribute("itemid");
  },  
            
  /**
   * Based on openGroupBookmark in bookmarks.js.
   */   
  openURLs: function nlv_openURLs(urls, where) {
 
    // get the top window and set the where to window if there is not a top
    var win = getTopWin();
    if (!win)
      where = "window";
    
    // confirm the open in tabs  
    if (where == "tab" && !this.confirmOpenURLs(urls.length))
      return;

    // do tab loading
    var url;           
    var branch = this.utils.getBranch("");      
    if (where == "current" || where == "tab" || where == "tabshifted") {
      var browser = win.document.getElementById("content");
      var tabPanels = browser.browsers;
      var tabs = browser.mTabContainer.childNodes;
      var tabCount = tabPanels.length;
      var doReplace = branch.getBoolPref("browser.tabs.loadFolderAndReplace");
      var loadInBackground = branch.getBoolPref("browser.tabs.loadBookmarksInBackground");
      var index0;
      
      // determine which tab to start loading in
      if (doReplace)
        index0 = 0;
      else {
        for (index0=tabCount-1; index0>=0; --index0) {
          var tab = tabPanels[index0];
          if (tab.webNavigation.currentURI.spec != "about:blank" ||
              tab.webProgress.isLoadingDocument)
            break;
        }
        ++index0;
      }
      
      // loop through the urls
      var index = index0;
      while (urls.length) {
        url = urls.shift();

        // load the url
        if (index < tabCount)
          tabPanels[index].loadURI(url);
        else
          browser.addTab(url);
        ++index;
      }
      
      // focus the tab if not load in background or is replace
      if (!loadInBackground || doReplace) {
        function selectNewForegroundTab(browser, tab) {
          browser.selectedTab = tab;
        }
        window.setTimeout(selectNewForegroundTab, 0, browser, tabs[index0]);
      }
      
      // remove excess tabs
      for (var i= tabCount-1; i>=index; --i)
        browser.removeTab(tabs[i]);
      
      // focus the window content  
      window.content.focus();
          
    // open the window with the urls
    } else if (where == "window")
      window.openDialog(getBrowserURL(), "_blank", "chrome,all,dialog=no", 
                        urls.join("|"));  
  }, 
  
  confirmOpenURLs: function nlv_confirmOpenUrls(count) {
  
    // don't ask for the confirmation
    var branch = this.utils.getBranch("");
    var dontAsk = !branch.getBoolPref("browser.tabs.warnOnOpen");
    if (dontAsk) 
      return true;
  
    // count is less then number needed to show the confirmation
    if (count < branch.getIntPref("browser.tabs.maxOpenBeforeWarn"))
      return true;

    // open the confirmation message
    var warnOnOpen = { value: true };
    var pmService = Cc["@mozilla.org/embedcomp/prompt-service;1"].
                    getService(Ci.nsIPromptService);
                     
    const PREFIX = "newslive.utils.tabs.";
    var bundle = document.getElementById("bundle_brand");
    var shortName = bundle.getString("brandShortName");   
    var title = this.utils.getString(PREFIX + "title");
    var message = this.utils.getString(PREFIX + "message", 
                                       [count, shortName]);
    var show = this.utils.getString(PREFIX + "show", [shortName]);
    var button = this.utils.getString(PREFIX + "button");       
    var flags = (pmService.BUTTON_TITLE_IS_STRING * pmService.BUTTON_POS_0) +
                (pmService.BUTTON_TITLE_CANCEL * pmService.BUTTON_POS_1);    
    var open = pmService.confirmEx(window, title, message, flags, button, null, 
                                   null, show, warnOnOpen);
    var reallyOpen = (open === 0);  

    // set the don't ask preference
    if (reallyOpen && !warnOnOpen.value)
      branch.setBoolPref("browser.tabs.warnOnOpen", false);

    // return the users results
    return reallyOpen;
  },

  getNode: function nlv_getNode(itemid, suffix) {
    return document.getElementById(itemid + suffix);
  },
  
  createNode: function nlv_createNode(item, parent) {
  
    // determine the type of item
    var itemtype;
    if (this.isFeed(item))
      itemtype = "feed";
    else
      itemtype = "item";
    
    // feed 
    var node, child, row, cell;
    if (itemtype == "feed") {
    
      // create a menu
      if (this.type == "menu") { 
        node = document.createElementNS(kXULNS, "menu");
        node.setAttribute("id", item.id + "_menu");
        node.setAttribute("class", "menu-iconic newslive-item");
        node.setAttribute("label", item.title);
        node.setAttribute("image", item.iconURL);   
        node.setAttribute("crop", "end"); 
        child = document.createElementNS(kXULNS, "menupopup");
        child.setAttribute("id", item.id + "_menupopup");
        child.setAttribute("context", "newslive_context");
        child.setAttribute("crop", "end");
        node.appendChild(child);
      
      // create a tree row
      } else {
        node = document.createElementNS(kXULNS, "treeitem");
        node.setAttribute("id", item.id + "_treeitem");
        node.setAttribute("class", "newslive-item");
        node.setAttribute("container", "true");
        node.setAttribute("label", item.title);
        row = document.createElementNS(kXULNS, "treerow");
        row.setAttribute("id", item.id + "_treerow");
        cell = document.createElementNS(kXULNS, "treecell");
        cell.setAttribute("id", item.id + "_treecell");
        cell.setAttribute("label", item.title);
        cell.setAttribute("src", item.iconURL);
        cell.setAttribute("crop", "end");
        child = document.createElementNS(kXULNS, "treechildren");
        child.setAttribute("id", item.id + "_treechildren");
        row.appendChild(cell);
        node.appendChild(row);
        node.appendChild(child);    
      }
      
      // set attributes common to both
      node.setAttribute("itemid", item.id);
      node.setAttribute("itemtype", itemtype);
      node.setAttribute("url", item.URL);      
      node.setAttribute("sortIndex", String(item.sortIndex));
      
      // add the node to the document and return it
      this.addNode(node, parent);
      return child;      
    }
    
    
    // item   
    // create a menuitem      
    var disabled = (item.URL == "about:newslive");
    if (this.type == "menu") {
      node = document.createElementNS(kXULNS, "menuitem");
      node.setAttribute("id", item.id + "_menuitem");
      node.setAttribute("class", "menuitem-iconic newslive-item");
      node.setAttribute("label", item.title); 
      node.setAttribute("crop", "end");
      
    // create a tree row     
    } else {
      var properties = "";
      if (disabled)
        properties = "disabled";   
      node = document.createElementNS(kXULNS, "treeitem");
      node.setAttribute("id", item.id + "_treeitem");
      node.setAttribute("class", "newslive-item");
      node.setAttribute("label", item.title);      
      row = document.createElementNS(kXULNS, "treerow");
      row.setAttribute("id", item.id + "_treerow");
      cell = document.createElementNS(kXULNS, "treecell"); 
      cell.setAttribute("id", item.id + "_treecell");            
      cell.setAttribute("label", item.title);
      cell.setAttribute("properties", properties); 
      cell.setAttribute("crop", "end");     
      row.appendChild(cell);
      node.appendChild(row);
    }
    
    // set common attributes
    node.setAttribute("itemtype", "item");    
    if (disabled)
      node.setAttribute("disabled", "true");
    else {    
      node.setAttribute("itemid", item.id);
      node.setAttribute("url", item.URL);
      node.setAttribute("statustext", item.URL);
    }
    node.setAttribute("sortIndex", String(item.sortIndex));
    
    // add to the document
    this.addNode(node, parent);
    
    // return it
    return node;
  },
        
  addNode: function nlv_addNode(node, parent) {
  
    // append since there are no children
    if (!parent.hasChildNodes()) {
      parent.appendChild(node);
      return;
    }

    // append since we are at the last child
    var child = parent.lastChild;
    if (child == this.lastChild) {
      parent.appendChild(node);
      return;
    }
    
    // append if the sortIndex is greater then the last childs
    var childIndex = Number(child.getAttribute("sortIndex"));
    var sortIndex = Number(node.getAttribute("sortIndex"));
    if (sortIndex >= childIndex) {
      parent.appendChild(node);
      return;
    }
   
    // determine the next child to check
    child = this.lastChild;
    if (!child)
      child = parent.firstChild;
    else if (child.parentNode != parent)
        child = parent.firstChild;
    else if (child.nextSibling)
      child = child.nextSibling;
      
    // just append since we could not find the correct child
    else {
      parent.appendChild(node);
      return;
    }
    
    // insert before if the sortIndex is less than the childs  
    childIndex = Number(child.getAttribute("sortIndex"));
    if (sortIndex <= childIndex) {
      parent.insertBefore(node, child);
      return;
    }
    
    // loop through the child nodes
    while (child) {
    
      // get the next sibling
      var next = child.nextSibling;
      if (!next)
        break;
    
      // insert in between if sortIndex is in between the two nodes
      childIndex = Number(child.getAttribute("sortIndex"));
      var nextIndex = Number(next.getAttribute("sortIndex"));
      if (childIndex <= sortIndex && nextIndex >= sortIndex) {
        parent.insertBefore(node, next);
        return;
      }
      
      // set the child to the next sibling
      child = next;
    }
    
    // could not find the correct position so just append 
    parent.appendChild(node);
  },
      
  removeNode: function nlv_removeNode(node) {
    node.parentNode.removeChild(node);
  },
   
  addLoading: function nlv_addLoading(node, type) {
    var create = false;
    
    // determine the child to check
    var child;
    if (!this.lastChild)
      child = node.firstChild;
    else
      child = node.lastChild;
    
    // no child so create
    if (!child)
      create = true;
      
    // child is the last one so create
    else if (child == this.lastChild)
      create = true;
      
    // child is already the loading message do not create
    else if (child.getAttribute("class") == "loading-menuitem")
      create = false;
      
    // do not create
    else
      create = false;
    
    // create loading message
    if (create) {
    
      // create a menuitem
      var label = this.utils.getString("newslive.loading." + type);
      if (this.type == "menu") {
        child = document.createElementNS(kXULNS, "menuitem");
        child.setAttribute("label", label);
        
      // create a tree row
      } else {
        child = document.createElementNS(kXULNS, "treeitem");
        var row = document.createElementNS(kXULNS, "treerow");
        var cell = document.createElementNS(kXULNS, "treecell");
        cell.setAttribute("properties", "disabled");
        cell.setAttribute("label", label);
        row.appendChild(cell);
        child.appendChild(row);
      }
      
      // set common attributes
      child.setAttribute("class", "loading-menuitem");
      child.setAttribute("disabled", "true");
      
      // add to the document
      node.appendChild(child);
    }
  },
  
  removeLoading: function nlv_removeLoading(node) {
    // no children so do not remove
    if (!node.hasChildNodes())
      return;

    // remove the first child if is loading
    if (node.firstChild.getAttribute("class") == "loading-menuitem")
      this.removeNode(node.firstChild);
      
    // remove the last child if is loading
    else if (node.lastChild.getAttribute("class") == "loading-menuitem")
      this.removeNode(node.lastChild);
  },
  
  addOpenInTabs: function nlv_addOpenInTabs(node) {
    // do not add if the view is a tree
    if (this.type == "tree")
      return;
    
    // do not add if not valid open in tabs  
    if (!this.isValidOpenInTabs(node))
      return;
    
    // do not add if already added
    var child = node.lastChild;
    if (child.getAttribute("class") == "openintabs-menuitem")
      return;
    
    // create a menuseparator  
    child = document.createElementNS(kXULNS, "menuseparator");
    child.setAttribute("class", "openintabs-menuseparator");
    node.appendChild(child);
    
    // create the menuitem
    const PREFIX = "newslive.utils.tabs.";
    child = document.createElementNS(kXULNS, "menuitem");
    child.setAttribute("class", "openintabs-menuitem");
    child.setAttribute("label", this.utils.getString(PREFIX + "label"));
    child.setAttribute("accesskey", this.utils.getString(PREFIX + "accesskey"));
    
    // add to the document
    node.appendChild(child);
  },
  
  removeOpenInTabs: function nlv_removeOpenInTabs(node) {
    // no children so do not remove
    if (!node.hasChildNodes())
      return;
    
    // loop through child nodes
    var child = node.lastChild;
    var removed = 0;
    while (child) {

      // open item
      var cclass = child.getAttribute("class");
      if (cclass == "openintabs-menuitem" || 
          cclass == "openintabs-menuseparator") {
      
        // remove node
        var previous = child.previousSibling;
        this.removeNode(child);
        child = previous;
        removed++;
        
        // all removed so break
        if (removed == 2)
          break;
          
      // get the next child
      } else 
        child = child.previousSibling;
    }
  },
  
  isValidOpenInTabs: function nlv_isValidOpenInTabs(node) {
    // no children so not valid
    if (!node.hasChildNodes())
      return false;

    // loading message not valid
    var child = node.firstChild;
    if (child.getAttribute("class") == "loading-menuitem")
      return false;
    
    // loop through children    
    var count =0;
    do {
    
      // at least 2 items so is valid
      ++count;
      if (count == 2)
        return true;
        
      // get the next child  
      child = child.nextSibling;
    } while (child);
    
    // less than two items
    return false;
  }
};