/*******************************************************************************
 * Main object that interacts between the user and the backend.
 ******************************************************************************/
var Sitemail = {
  
  // nsISupports
  QueryInterface: function sml_QI(iid) {
    if (iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
  
  observe: function sml_observe(subject, topic, data) {

    if (topic != "netscape-sitemail")
      return;
      
    // do update requested by back-end
    var self = this;  
    window.setTimeout(function() { self.update(); }, 0);
  },
  
  get button() { return document.getElementById("sitemail_button"); },
  get command() { return document.getElementById("sitemail_cmd"); },
  get menuitem() { return document.getElementById("sitemail_menuitem"); },
  get currentURL() { return window.content.document.location.href; },
    
  _domain: "extensions.netscape.sitemail.",
  _branch: null,
  get branch() { return this._branch; },
  set branch(val) { this._branch = val; },
  
  _obService: null,
  get obService() { return this._obService; },
  set obService(val) { this._obService = val; },
  
  _sitemailService: null,
  get sitemailService() { return this._sitemailService; },
  set sitemailService(val) { this._sitemailService = val; },
  
  _loaded: false,
  get loaded() { return this._loaded; },
  set loaded(val) { this._loaded = val; },
  
  onLoad: function sml_onLoad() {
    // already loaded
    if (this.loaded)
      return;
    this.loaded = true;
        
    // Get the sitemail service
    this.sitemailService = Cc["@netscape.com/sitemail/service;1"].
                      getService(Ci.nsISitemailService);    
                          
    // Get the observer service and add ourself as an observer
    this.obService = Cc["@mozilla.org/observer-service;1"].
                     getService(Ci.nsIObserverService);
    this.obService.addObserver(this, "netscape-sitemail", false);
    
    // get the pref branch
    this.branch = Cc["@mozilla.org/preferences-service;1"].
                  getService(Ci.nsIPrefService).
                  getBranch(this._domain);
          
    // Update the UI
    var self = this;
    window.setTimeout(function() { self.update(); }, 100);                      
  },
  
  onUnload: function sml_onUnload() {
    // never loaded
    if (!this.loaded)
      return;
    this.loaded = false;
        
    // remove ourself as an observer
    this.obService.removeObserver(this, "netscape-sitemail");

    // remove variables
    this.obService = null;
    this.sitemailService = null;  
    this.branch = null;   
  },
  
  getString: function sml_getString(name, replace) { 
  
    // get the string bundle
    var bundle = document.getElementById("sitemail_bundle");
    if (!bundle)
      return null;
    
    // return the string  
    if (!replace)
      return bundle.getString(name);
    else
      return bundle.getFormattedString(name, replace);
  },
      
  update: function sml_update() {

    // set disabled attribute on the command and button
    var disabled = this.branch.getBoolPref("disabled");
    var button = this.button;
    var command = this.command;
    var menuitem = this.menuitem;
    if (disabled) {
      command.setAttribute("hidden", "true");
      command.setAttribute("disabled", "true");
      menuitem.setAttribute("hidden", "true");
      menuitem.setAttribute("disabled", "true");      
      if (button) {
        button.setAttribute("hidden", "true");
        button.setAttribute("disabled", "true");
      }
    } else {
      command.removeAttribute("hidden");
      command.removeAttribute("disabled");    
      menuitem.removeAttribute("hidden");
      menuitem.removeAttribute("disabled");
      if (button) {
        button.removeAttribute("hidden");
        button.removeAttribute("disabled");
      }
    }
    
    // Get the sitemail Button
    if (!button || button.hasAttribute("disabled"))
      return;

    // Update with paused
    const SML_PREFIX = "sitemail.overlay.tooltip.";   
    button.removeAttribute("new");     
    if (this.sitemailService.paused) {
      command.setAttribute("disabled", "true");    
      menuitem.setAttribute("disabled", "true");              
      button.setAttribute("disabled", "true"); 
      button.setAttribute("tooltiptext", this.getString(SML_PREFIX + "paused"));

    // Update with not signed in    
    } else if (!this.sitemailService.isSignedIn) {
      button.setAttribute("tooltiptext", this.getString(SML_PREFIX + "login"));
    
    // Update with no activity
    } else if (!this.sitemailService.hasMail) {
      button.setAttribute("tooltiptext", this.getString(SML_PREFIX + "none"));

    // Update with activity
    } else {
      var count = String(this.sitemailService.totalCount);
      button.setAttribute("new", "true");
      if (count == "1")
        button.setAttribute("tooltiptext", this.getString(SML_PREFIX + "single", [count]));
      else
        button.setAttribute("tooltiptext", this.getString(SML_PREFIX + "plural", [count]));
    }  
  },
  
  checkActivity: function sml_checkActivity() {
    this.sitemailService.checkActivity();
  },
  
  checkURL: function sml_checkURL(URL) {
    if (!this.loaded)
      this.onLoad();  
    this.sitemailService.checkURL(URL);
  },
    
  open: function sml_open(event) {
    var URL = this.sitemailService.URL;
    var where = nsWhereToOpenLink(this._domain + "open", event);
    openUILinkIn(URL, where);
  }
};